// program to display and calibrate the ATtiny85 OSCCAL register on the ATtiny85 pulser
// this sketch typically runs on an Arduino UNO
// pin 13 is used to supply power to the ATtiny85pulser, and the pulser's row of input/motor pins are plugged into the UNO's pins 8 to 12 as follows
#define POWER 13
#define MOTOR_1 8
#define MOTOR_2 9
#define AUX 10
#define ELE 11
#define RUD 12
// the pulser's GND must, of course, be linked to the UNO's GND

uint32_t previousMicros, previousMillis;
void setup() {
  digitalWrite(POWER, LOW);
  pinMode(POWER, OUTPUT); // prepare power pin, but don't power the pulser up yet
  delay(2000); // give the pulser time to power down
  
  // the pulser enters OSCCAL configuration mode (its configuration 5) when it sees its RUD and AUX input pins linked to its ELE output pin
  // so we need to set our ELE as an input, and copy its state to our RUD and AUX outputs for a while after power up
  pinMode(ELE, INPUT);
  pinMode(RUD, OUTPUT);
  pinMode(AUX, OUTPUT);

  uint32_t endOfTestMillis = millis() + 1000;
  digitalWrite(POWER, HIGH); // power up pulser
  while (millis() < endOfTestMillis) {
    int state = digitalRead(ELE);
    digitalWrite(RUD, state);
    digitalWrite(AUX, state);
  }
  digitalWrite(RUD, LOW);
  pinMode(RUD, INPUT);
  digitalWrite(AUX, LOW);
  pinMode(AUX, OUTPUT);

  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);

  Serial.begin(115200);
  previousMillis = millis();
  previousMicros = micros();
}

uint32_t freq; // smoothed frequency // determined from CLK transmitted by pulser on ELE pin
uint8_t osccal = 0xFF; // osccal value of pulser, transmitted serially on RUD pin
bool firstPass = true;
uint32_t tmr2, tmr3, tmr4;
void loop() {
  static uint8_t receiveByte, receiveMode = 0;
  static uint8_t transmitByte, transmitMode = 0;
  while(digitalRead(ELE) == LOW) {} // wait for positive edge on ELE
  // transmit any waiting bytes on AUX, changing state on positive edges
  if (transmitMode) {
    if (transmitMode == 1) { // send start bit
      digitalWrite(AUX, HIGH);
    } else if (transmitMode < 10) { // sending bits of transmitByte
      if (transmitByte & 0x80) {
        digitalWrite(AUX, HIGH);
      } else {
        digitalWrite(AUX, LOW);
      }
      transmitByte <<= 1;
    } else { // send stop bit
      digitalWrite(AUX, LOW);
    }
    if (++transmitMode > 10) {
      transmitByte = transmitMode = 0; // back to idle
    }
  }
  while(digitalRead(ELE) == HIGH) {} // wait for negative edge on ELE
  uint32_t now = micros();
  // bits change on positive edge and are read on negative edge
  uint8_t receiveState = 0;
  if (digitalRead(RUD) == HIGH) {
    receiveState = 0x01;
  }
  if (!receiveMode) {
    if (receiveState) { // start bit received
      receiveMode = 1;
      receiveByte = 0;
    }
  } else {
    if (receiveMode <= 8) { // receiving bits, MSB first
      receiveByte = (receiveByte << 1) | receiveState;
      receiveMode++;
    } else if (receiveMode == 9) { // expecting stop bit
      if (!receiveState) { // stop bit received
        osccal = receiveByte; // store received value
      }
      receiveMode = 0; // back to idle mode
    }
  }


  uint32_t elapsed = now - previousMicros;
  previousMicros = now;
  uint32_t sampleFreq = 64000000UL / elapsed;
  if (firstPass) {
    firstPass = false;
  } else {
    freq -= (freq >> 8);
    freq += sampleFreq;
  }
  now = millis();
  elapsed = now - previousMillis;
  if (elapsed > 500) {
    previousMillis += 500;
    //Serial.print(sampleFreq);
    //Serial.print(" ");
    Serial.print(freq >> 14);
    Serial.print(" Hz");
    if (osccal != 0x0FF) {
      Serial.print(" OSCCAL = 0x");
      Serial.print(osccal, HEX);
    }
    Serial.print("\n");
  }
  // debounce digital inputs 2, 3, and 4
  int pin2 = digitalRead(2);
  if (!tmr2) { // timer expired - ready to detect edge
    if (pin2 == LOW) { // pressed
      if (transmitMode == 0) {
        transmitByte = '+';
        transmitMode = 1;
        Serial.println("+");
      }
      tmr2 = 100; // start timer
    }
  } else { // run timer once input released
    if (pin2 == LOW) {
      tmr2 = 100;
    } else {
      if (tmr2 > elapsed) {
        tmr2 -= elapsed;
      } else {
        tmr2 = 0;
      }
    }
  }

  int pin3 = digitalRead(3);
  if (!tmr3) { // timer expired - ready to detect edge
    if (pin3 == LOW) { // pressed
      if (transmitMode == 0) {
        transmitByte = '-';
        transmitMode = 1;
        Serial.println("-");
      }
      tmr3 = 100; // start timer
    }
  } else { // run timer once input released
    if (pin3 == LOW) {
      tmr3 = 100;
    } else {
      if (tmr3 > elapsed) {
        tmr3 -= elapsed;
      } else {
        tmr3 = 0;
      }
    }
  }

  int pin4 = digitalRead(4);
  if (!tmr4) { // timer expired - ready to detect edge
    if (pin4 == LOW) { // pressed
      if (transmitMode == 0) {
        transmitByte = 'W';
        transmitMode = 1;
        Serial.println("W");
      }
      tmr4 = 100; // start timer
    }
  } else { // run timer once input released
    if (pin4 == LOW) {
      tmr4 = 100;
    } else {
      if (tmr4 > elapsed) {
        tmr4 -= elapsed;
      } else {
        tmr4 = 0;
      }
    }
  }
}
